/*
 */

/* Gray color means all RGB values are the same as graph value in original graph.
 */
function gray(v) { 
	// R, G, B are all equals in original graph.
	blue = v & 0xff;
	return blue;
}

function dist(x0, y0, x1, y1) {	return sqrt( (x0 - x1) * (x0 - x1) + (y0 - y1) * (y0 - y1) );}
function distTriangle(a, b) { return sqrt(a * a + b * b); }
function PtoLine(x0, y0, x1, y1, x2, y2, lineLen) {
	return abs( (x2 - x1) * (y1 - y0) - (x1 - x0) * (y2 - y1) ) / lineLen;
}
function vectorMul(x1, y1, x2, y2) {
	return x1 * y2 - x2 * y1;
}
function max1(a, b) { if (a > b) return a; return b; };
function min1(a, b) { if (a > b) return b; return a; };

/* 
 *  Try to find the Max point (store in c) and return it.
 *  	(x1, y1) is the last point, (x2, y2) is -10 points that used for slope.
 *  Method optional:
 *  1) Using diagonal method.
 *  2) tracking tip?
 */
function getMax(x1, y1, x2, y2, width, height, point) {
	lineLen = 20; // ??
	d = dist(x1, y1, x2, y2);
	
	//write(x1, y1, x2, y2, d);
	x2 = x1 + ((x2 - x1) * lineLen) / d;
	y2 = y1 + ((y2 - y1) * lineLen) / d;
	if (y2 == y1) {	// slope cannot zero
		y2 = y1 + 0.00001;
	}
	//write("New X2, y2:", x2, y2);
	
	maxLen = distTriangle(bias, lineLen);
	// PtoLine = abs( (x2 - x1) * (y1 - y0) - (x1 - x0) * (y2 - y1) ) / lineLen;
	
	//write("MaxLen:", maxLen);

	slope = (x2 - x1) / (y1 - y2);
	bbb = y1 - (x1 * slope);
	//write("slope, b :", slope, bbb);
	
	p1x = x1 - bias;
	p2x = x1 + bias;
	p1y = p1x * slope + bbb;
	p2y = p2x * slope + bbb;

	//write("P1, P2:", p1x, p1y, p2x, p2y);

	// vectorMul( (P2 - P0) * (P1 - P0) )
	
	ret = -1;
	for (y0 = y1 + maxLen; y0 > y1 - maxLen; y0--) {	// find last
		if (y0 > 0 && y0 < height) {
			for (x0 = min1(x1, x1) - maxLen; x0 < max1(x1, x1) + maxLen; x0++) {
				vec = vectorMul(p2x - x0, p2y - y0, p1x - x0, p1y - y0);
				if ((x0 > 0 && x0 < width) && 
					//(dist(x0, y0, x1, y1) < maxLen) &&
					//(dist(x0, y0, x2, y2) < maxLen) &&
					(abs((x2 - x1) * (y1 - y0) - (x1 - x0) * (y2 - y1)) / lineLen <  bias) &&
					vec <= 0
				     )
				{
					v = getPixel(x0, y0);
					t1 = gray(v);
					if (t1 > cutOff) {
						//write("DEBUG: x0, y0 :", x0, y0);
						//write("Vector:", vec);
						//write("DEBUG:", dist(x0, y0, x1, y1));
						//write("DEBUG:", dist(x0, y0, x2, y2));
						//write("DEBUG:", abs((x2 - x1) * (y1 - y0) - (x1 - x0) * (y2 - y1)) / lineLen);
						
						setColor(0, 0, 200);
						drawLine(p1x, p1y, p2x, p2y);					
						
						point[0] = x0;
						point[1] = y0;
						return t1;
					}
				}
			} // for
		} // if
	} // for
	return ret;
}

/*
function getMean(x, y, width, height) {
	//
	cc = 0;
	value = 0;
	bias = 20;
	for (i = 0 - bias; i < bias; i++) {
		xx = x + i;
		if (xx > 0 && xx < width) {
			for (j = 0 - bias; j < bias; j++) {
				cc++;
				yy = y + j;
				if (yy > 0 && yy < width) {
					value += getPixel(xx, yy);
				}
			}
		}
	}
	if (cc == 0) {
		return 999999999;
	}
	return (value / cc);
}
*/

/*
 * Main function to find the Tip. Each slice will call this once.
 * x, y : the user drawn line.
 */
function findTip(x, y, width, height, cutOff, tip) {
	j = -1;
	averageCount = 20;
	prevPosition = 30;
	for (i = x.length - 1; i >= prevPosition + averageCount; i--) { // at least 30 extra elements.
		// c is used to store the root tip
		c = newArray(2);
		c[0] = -1;
		c[1] = -1;

		// Use average to calculate the slope
		sumX = 0;
		sumY = 0;
		cc = 0;
		for (j = i - prevPosition; j >= i - prevPosition - averageCount; j--) { 
		//for (j = i - 50; j >= i - 70; j--) {
			sumX += x[j];
			sumY += y[j];
			cc++;
		}
		sumX /= cc;
		sumY /= cc;
		
		// max = getMax(startX, startY, sumX, sumY, width, height, c);
		max = getMax(x[i], y[i], sumX, sumY, width, height, c);
		
		if (max > cutOff) {
			write("Max Value, pos      :", max, i);
			j = i;
			i = -1;
			// write("Oval Position: ", c[-0], c[1]);
			
			setColor(255, 0, 0);	// red
			drawOval(c[0] - 2, c[1] - 2, 5, 5);
			tip[0] = c[0] - 2;
			tip[1] = c[1] - 2;
			
			//drawOval(startX - 1, startY - 1, 2, 2);
			//setColor(255, 255, 0);
			
			setColor(0, 255, 0);	// green
			drawOval(x[j] - 1, y[j] - 1, 2, 2);
		}
	}

	if (j == -1) { 	// did not find anything. no change.
		j = x.length;
	} else {
		j++;
	}
	write("Current, to Cut: ", x.length, x.length - j);
	return j;
}

macro "RootTip" {
	setTool("freeline");
	
	cutOff = 10;
	cutOff = getNumber("Cutoff, 0 is Black, 255 is White, use smaller number first, if cannot change it: ", cutOff);
	bias = 50;
	bias = getNumber("Scanner Line Length, if tip shacking largely, increase this number, else decrease it.", bias);
	if (bias < 10) {
		bias = getNumber("Scanner Line Length should longer than 10, are you sure use this length?", bias);
	}
	if (getBoolean("Clear the result?")) {
		IJ.deleteRows(0, nResults - 1);
	}

	
	type = selectionType();

	// x, y and sx, sy  stored the selected line.
	getSelectionCoordinates(sx, sy);
	write(sx.length);
	x = Array.copy(sx);
	y = Array.copy(sy);
	
	// we need to know only how many frames there are
	getDimensions(width, height, dummy, Slices, nFrames);
	write("SliceN:", Slices);
	write("Frame:", nFrames);

	useSlice = true;
	if (Slices == 1) {
		useSlice = false;
		Stack.getPosition(channelDummy, SliceDummy, curPos);
	} else {
		Stack.getPosition(channelDummy, curPos, FrameDummy);
	}
	
	//for (slice = 2 * (Slices / 3) ; slice >= 0 && i < 20; slice--) {
	tip = newArray(2);
	for (posIdx = curPos; posIdx >= 1; posIdx--) {
		// write("Slice________________:", slice);
		if (useSlice) {
			Stack.setSlice(posIdx);
		} else {
			Stack.setFrame(posIdx);
		}
		cut = findTip(x, y, width, height, cutOff, tip);
		
		x = Array.trim(x, cut);
		y = Array.trim(y, cut);
		
		makeSelection(7, x, y);
		run("Measure");
		
		setResult("Slice", nResults - 1, posIdx);
		setResult("TipX", nResults - 1, tip[0]);
		setResult("TipY", nResults - 1, tip[1]);
		if (posIdx != curPos) {
			v1 = getResult("Length", nResults - 2);
			v2 = getResult("Length", nResults - 1);
			setResult("Growth", nResults - 1, v1 - v2);
			
			tx = getResult("TipX", nResults - 2);
			ty = getResult("TipY", nResults - 2);
			setResult("TipGrowth", nResults - 1, dist(tx, ty, tip[0], tip[1]));
		} else {
			setResult("Growth", nResults - 1, 0);
			setResult("TipGrowth", nResults - 1, 0);
		}
	}
	write("All Finished");
	makeSelection(7, sx, sy);
}

// write(nResults);
// write(  getResult("Length", nResults - 1)  );

